#!/usr/bin/env node
"use strict";

// ------------------------------------------------------------------------
// Copyright (c) 2018-2024 Alexandre Bento Freire. All rights reserved.
// Licensed under the MIT License.
// ------------------------------------------------------------------------
var __spreadArray=this&&this.__spreadArray||function(to,from,pack){if(pack||2===arguments.length)for(var ar,i=0,l=from.length;i<l;i++)!ar&&i in from||(ar||(ar=Array.prototype.slice.call(from,0,i)),ar[i]=from[i]);return to.concat(ar||Array.prototype.slice.call(from))};Object.defineProperty(exports,"__esModule",{value:!0});var Cli,sysFs=require("fs"),sysPath=require("path"),sysProcess=require("process"),child_process_1=require("child_process"),opts_parser_js_1=require("../shared/opts-parser.js"),fsix_js_1=require("../shared/vendor/fsix.js"),version_js_1=require("../shared/version.js"),consts_js_1=require("../shared/lib/consts.js"),rel_consts_js_1=require("../shared/rel-consts.js"),http_server_ex_js_1=require("../shared/vendor/http-server-ex.js");!function(Cli){var DO_PRINT_USAGE=0,DO_RUN_COMMAND=1,DO_EXIT=2,DEFAULT_PORT=9e3,CMD_CHECK="check",CMD_CREATE="create",CMD_SERVE="serve",CMD_RENDER="render",CMD_GIF="gif",CMD_MOVIE="movie",DEFAULT_GIF_NAME="story.gif",DEFAULT_MOVIE_NAME="story.mp4",DEFAULT_BACKGROUND="white",MANUAL_BACKGROUND="manual",logLevel=consts_js_1.Consts.LL_ERROR,isVerbose=!1,cmdName="",cmdParam="",outArgs=[],isWin="win32"===sysProcess.platform,argOpts=opts_parser_js_1.OptsParser.argOpts;function runSpawn(cmdLine,args,callback){isVerbose&&(console.log("spawn cmdLine: ".concat(cmdLine)),console.log("args: ".concat(args.join(" "))));var ls=(0,child_process_1.spawn)(cmdLine,args);ls.stdout.on("data",function(data){logLevel>=consts_js_1.Consts.LL_SILENT&&console.log(data.toString())}),ls.stderr.on("data",function(data){logLevel>=consts_js_1.Consts.LL_SILENT&&console.error(data.toString())}),ls.on("close",function(_code){callback()})}function copyTree(srcPath,dstPath,onCopyText,allowCopy){isVerbose&&console.log("Copying Directory ".concat(srcPath," to ").concat(dstPath)),fsix_js_1.fsix.mkdirpSync(dstPath),sysFs.readdirSync(srcPath).forEach(function(fileBase){if(!allowCopy||allowCopy(fileBase)){var srcFileName="".concat(srcPath,"/").concat(fileBase),dstFileName="".concat(dstPath,"/").concat(fileBase),stats=sysFs.statSync(srcFileName);if(stats.isFile()){isVerbose&&console.log("Copying ".concat(srcFileName," to ").concat(dstFileName));var data=fsix_js_1.fsix.readUtf8Sync(srcFileName);onCopyText&&(data=onCopyText(data,fileBase)),sysFs.writeFileSync(dstFileName,data)}else stats.isDirectory()&&copyTree(srcFileName,dstFileName,onCopyText,allowCopy)}})}function getReport(){var reportFileName=fsix_js_1.fsix.toPosixSlash(cmdParam||"."),realReportFileName=sysPath.posix.join(reportFileName,"story-frames/frame-report.json");if(sysFs.existsSync(realReportFileName)&&(reportFileName=realReportFileName),isVerbose&&(console.log("reportFileName: ".concat(reportFileName)),console.log("realReportFileName: ".concat(realReportFileName))),!sysFs.existsSync(reportFileName))throw"Report file ".concat(reportFileName," doesn't exist");var report=fsix_js_1.fsix.loadJsonSync(reportFileName);return report.dirname=sysPath.dirname(reportFileName),isVerbose&&console.log("report path: ".concat(report.dirname)),"./"===report.framespattern.substr(0,2)&&(report.framespattern=report.dirname+"/"+report.framespattern.substr(2)),report}argOpts.port={param:"int",desc:"port for serve command. default is ".concat(DEFAULT_PORT)},argOpts.gif={param:"string",desc:"output gif filename. default is ".concat(DEFAULT_GIF_NAME)},argOpts.movie={param:"string",desc:"output movie filename. default is ".concat(DEFAULT_MOVIE_NAME)},argOpts.bkgMovie={param:"string",desc:"movie filename to be used as background to blend with transparent images"},argOpts.noPlugins={desc:"creates a project without plugins"},argOpts.noTypescript={desc:"creates a project without TypeScript files"},argOpts.listDir={desc:"serve command lists the directory contents if url has the querystring '?dir'"},argOpts.loop={param:"string",desc:"defines how many times a gif will loop. set to -1 if you don't won't it to loop"},argOpts.moviePre={param:"array",allowOption:!0,desc:"arguments to be passed to ffmpeg before the arguments passed by abeamer"},argOpts.moviePost={param:"array",allowOption:!0,desc:"arguments to be passed to ffmpeg after the arguments passed by abeamer"},argOpts.gifPre={param:"array",allowOption:!0,desc:"arguments to be passed to gif generator(convert) before the arguments passed by abeamer"},argOpts.gifPost={param:"array",allowOption:!0,desc:"arguments to be passed to gif generator(convert) after the arguments passed by abeamer"},argOpts.gifBackground={param:"string",desc:'background color used to replace the alpha channel in gif generation.\nif this is value is set to "'.concat(MANUAL_BACKGROUND,'" , no parameters relating are passed to the gif generator.\ndefault is ').concat(MANUAL_BACKGROUND)};try{!function(){switch(function(){for(var args=sysProcess.argv,argI=1;-1===args[argI-1].search(/abeamer/);)argI++;return(cmdName=args[argI++])?"--version"===cmdName?(console.log(version_js_1.VERSION),DO_EXIT):opts_parser_js_1.OptsParser.iterateArgOpts(!0,function(){return args[argI++]},function(option,value){switch(option){case"@param":return cmdParam=value,void outArgs.push(cmdParam);case"version":return console.log(version_js_1.VERSION),DO_EXIT;case"ll":(isVerbose=(logLevel=value)>=consts_js_1.Consts.LL_VERBOSE)&&(console.log("Args: [".concat(args.join("],["),"]")),console.log("Current Path: ".concat(sysProcess.cwd())))}switch(cmdName){case CMD_RENDER:outArgs.push("--"+option),value&&outArgs.push(value)}})||DO_RUN_COMMAND:DO_PRINT_USAGE}()){case DO_PRINT_USAGE:console.log("abeamer [command] [options] [project-name|report-name]\nThe commands are:\n    ".concat(CMD_CHECK," checks if the all requirements are installed and configured\n    ").concat(CMD_CREATE," creates a project with project-name\n    ").concat(CMD_SERVE,"  starts a live server. Use it in case you need to load the config from JSON file\n    ").concat(CMD_RENDER," runs your project in the context of the headless browser.\n    ").concat(CMD_GIF,"    creates an animated gif from the project-name or report-name\n    ").concat(CMD_MOVIE,'  creates a movie from the project-name or report-name\n\n    e.g.\n      echo "checks if chrome, puppeteer, imagemagick, ffmpeg are installed and configured"\n      abeamer ').concat(CMD_CHECK,'\n\n      echo "create folder foo and copy necessary files"\n      abeamer ').concat(CMD_CREATE,' --width 640 --height 480 --fps 25 foo\n\n      cd foo\n\n      echo "start a live server"\n      echo "only required if you need to load your configuration from json file"\n      abeamer ').concat(CMD_SERVE,'\n\n      echo "generates the png files and a report on story-frames folder"\n      abeamer ').concat(CMD_RENDER,'\n\n      echo "creates story.gif file on story-frames folder"\n      abeamer ').concat(CMD_GIF,'\n\n      echo "creates story.mp4 file on story-frames folder"\n      abeamer ').concat(CMD_MOVIE,"\n\n      For more information, read:\n      https://abeamer.devtoix.com/docs/latest/end-user/en/site/abeamer-cli/\n\n")),opts_parser_js_1.OptsParser.printUsage();break;case DO_RUN_COMMAND:switch(isVerbose&&console.log("Run Command: ".concat(cmdName)),cmdName){case CMD_CHECK:!function(){var checkCount=0,TOTAL_CHECK_COUNT=5;function displayCheck(what,passed,failedMsg){checkCount++,console.log("".concat(checkCount,". Check: ").concat(what," --\x3e ").concat(passed?"OK":"Failed")),passed||console.log("  TODO:"+failedMsg+"\n"),checkCount===TOTAL_CHECK_COUNT&&console.log("\n")}function addToStartUp(passed,key,value,failedMsg){displayCheck("".concat(key,"=").concat(value),passed,"\n    ".concat(failedMsg,"\n    Add to the shell startup script:\n").concat(isWin?"SET":"export"," ").concat(key,"=").concat(value))}function checkProgramIsValid(envKey,appName,versionParam,matchRegEx,requireMsg){function displayResult(passed){displayCheck(appName,passed,"\n    ABeamer requires ".concat(requireMsg,"\n    Either add the executable to the system path, or add to the shell startup script:\n").concat(isWin?"set":"export"," ").concat(envKey,"=<absolute-path-to-").concat(appName,">"))}var envValue=sysProcess.env[envKey];envValue?displayResult(sysFs.existsSync(envValue)):fsix_js_1.fsix.runExternal("".concat(appName," ").concat(versionParam),function(_error,stdout,stderr){displayResult(""===stderr&&stdout.match(matchRegEx))})}console.log("\nChecking:\n"),addToStartUp("true"===(sysProcess.env.PUPPETEER_SKIP_CHROMIUM_DOWNLOAD||"").toLowerCase(),"PUPPETEER_SKIP_CHROMIUM_DOWNLOAD","TRUE","by default puppeteer downloads Chromium which is doesn't support many features");var puppeteer,chromeBin=sysProcess.env.CHROME_BIN;chromeBin?addToStartUp(sysFs.existsSync(chromeBin),"CHROME_BIN","<chrome-path>","CHROME_BIN points to a missing chrome executable"):addToStartUp(!1,"CHROME_BIN","<chrome-path>","puppeteer uses by default Chromium, set CHROME_BIN with the absolute path to chrome web browser executable"),checkProgramIsValid("FFMPEG_BIN","ffmpeg","-version",/ffmpeg version/,"ffmpeg to generate movies"),checkProgramIsValid("IM_CONVERT_BIN","convert","--version",/Version: ImageMagick/,"ImageMagick convert program to generate gifs");try{puppeteer=require("puppeteer")}catch(error){}displayCheck("puppeteer",puppeteer,"\n   ABeamer requires puppeteer. Install using the following command\nnpm i puppeteer")}();break;case CMD_CREATE:!function(){var projName=fsix_js_1.fsix.toPosixSlash(cmdParam);if(!projName)throw"Missing project name";if("-"===projName[0]||-1!==projName.search(/[\"\'\?\*\+]/)||-1!==projName.search(/^[\.\/]+$/))throw"Error: ".concat(projName," is not valid project name");if(sysFs.existsSync(projName))throw"Error: Project ".concat(projName," already exists");if(projName.includes("/")){var dirname=sysPath.posix.dirname(projName);fsix_js_1.fsix.mkdirpSync(dirname)}var ROOT_PATH=fsix_js_1.fsix.toPosixSlash(__dirname)+"/..",TEMPLATE_PATH=ROOT_PATH+"/gallery/hello-world",LIB_PATH=ROOT_PATH+"/client/lib",width=argOpts.width.value||rel_consts_js_1.RelConsts.DEFAULT_WIDTH,height=argOpts.height.value||rel_consts_js_1.RelConsts.DEFAULT_HEIGHT,fps=argOpts.fps.value||rel_consts_js_1.RelConsts.DEFAULT_FPS,noPlugins=argOpts.noPlugins.hasOption,noTypescript=argOpts.noTypescript.hasOption;copyTree(TEMPLATE_PATH,projName,function(text,fileName){switch(sysPath.basename(fileName)){case"main.js":case"main.ts":noTypescript&&(text=text.replace(/^.*sourceMappingURL=.*$/m,"")),text=text.replace(/createStory\([^)]*\)/,"createStory(/*FPS:*/".concat(fps,")"));break;case"abeamer.ini":text=text.replace(/width:\s*\d+/,"width: ".concat(width)).replace(/height:\s*\d+/,"height: ".concat(height));break;case"main.min.css":text=text.replace(/.abeamer-scene{width:\d+px;height:\d+px}/,".abeamer-scene{width:".concat(width,"px;height:").concat(height,"px}"));break;case"index.html":if(!noPlugins){var plugins_1=fsix_js_1.fsix.loadJsonSync("".concat(ROOT_PATH,"/client/lib/plugins/plugins-list.json")),pre_1="",post_1="";text.replace(/^(.*)js\/abeamer\.min\.js(.*)$/m,function(_app,_pre,_post){return pre_1=_pre,post_1=_post,""}),text=text.replace(/^(.*js\/main\.js.*)$/m,function(all){return"\n          \x3c!-- remove the unnecessary plugins --\x3e\n"+plugins_1.map(function(plugin){return"".concat(pre_1,"plugins/").concat(plugin,"/").concat(plugin,".js").concat(post_1)}).join("\n")+"\n\n"+all})}}return text},function(fileBase){return!noTypescript||!fileBase.match(/(?:js\.map|\.ts|tsconfig\.json)$/)}),copyTree(LIB_PATH,"".concat(projName,"/abeamer"),void 0,function(fileBase){return!(noTypescript&&fileBase.match(/(?:typings|\.ts)$/)||noPlugins&&fileBase.match(/plugins$/)||fileBase.match(/plugins-list\.json$/))}),logLevel>consts_js_1.Consts.LL_SILENT&&console.log("Project ".concat(projName," created.\n- frame-width: ").concat(width,"px\n- frame-height: ").concat(height,"px\n- fps: ").concat(fps,"\nTo modify the the frame dimensions, edit [abeamer.ini] and recompile the [css/main.scss] file.\nTo modify the fps, edit the [js/main.ts] file.\n"))}();break;case CMD_SERVE:hasMarked=sysFs.existsSync(sysPath.posix.join(__dirname,"../node_modules/marked/bin/marked")),hasHighlightJs=sysFs.existsSync(sysPath.posix.join(__dirname,"../node_modules/highlight.js/lib/index.js")),port=argOpts.port.value||DEFAULT_PORT,allowDirListing=argOpts.listDir.hasOption,new http_server_ex_js_1.HttpServerEx.ServerEx(port,isVerbose,"EXIT_SERVER",allowDirListing,hasMarked,hasHighlightJs).start(),logLevel>=consts_js_1.Consts.LL_SILENT&&(hasMarked&&console.log("Using markdown compiler"),console.log("Serving on http://localhost:".concat(port,"/")),allowDirListing&&console.log("Directory listing on http://localhost:".concat(port,"/?dir")));break;case CMD_RENDER:!function(){var serverName=(opts_parser_js_1.OptsParser.argOpts.server.value||rel_consts_js_1.RelConsts.DEFAULT_SERVER).toLowerCase();if(-1===rel_consts_js_1.RelConsts.SUPPORTED_SERVERS.indexOf(serverName))throw"Unknown ".concat(serverName);cmdParam||argOpts.config.value||argOpts.url.value||outArgs.push(".");var configArgIndex=outArgs.indexOf("--config");-1!==configArgIndex&&(outArgs[configArgIndex]=outArgs[configArgIndex]+"__2"),outArgs.splice(0,0,"".concat(fsix_js_1.fsix.toPosixSlash(__dirname),"/../server/server-agent-").concat(serverName,".js")),runSpawn(argOpts.serverExec.value||(-1===rel_consts_js_1.RelConsts.NODE_SERVERS.indexOf(serverName)?serverName:"node"),outArgs,function(){logLevel>consts_js_1.Consts.LL_SILENT&&console.log("Server finished")})}();break;case CMD_GIF:!function(){var report=getReport(),gifFileName=argOpts.gif.value||"".concat(report.dirname,"/").concat(DEFAULT_GIF_NAME),cmdLine=sysProcess.env.IM_CONVERT_BIN||"convert",args=["-delay","1x".concat(report.fps)],scale=opts_parser_js_1.OptsParser.computeScale(report.width,report.height);scale&&args.push("-scale",scale.join("x"));var loop=argOpts.loop.value||"0";args.push("-loop",loop),args.push("-strip","-layers","optimize");var gifBackground=argOpts.gifBackground.value||DEFAULT_BACKGROUND;gifBackground!==MANUAL_BACKGROUND&&args.push("-background",gifBackground,"-alpha","remove"),args.push(report.framespattern.replace(/\%\d*d/,"*"),gifFileName),argOpts.gifPre.multipleValue&&(args=__spreadArray(__spreadArray([],argOpts.gifPre.multipleValue,!0),args,!0)),argOpts.gifPost.multipleValue&&(args=__spreadArray(__spreadArray([],args,!0),argOpts.gifPost.multipleValue,!0)),isVerbose&&console.log("\n".concat(cmdLine," ").concat(args.join(" "),"\n")),runSpawn(cmdLine,args,function(){logLevel>consts_js_1.Consts.LL_SILENT&&console.log("Created gif ".concat(gifFileName))})}();break;case CMD_MOVIE:!function(){var report=getReport(),movieFileName=argOpts.movie.value||"".concat(report.dirname,"/").concat(DEFAULT_MOVIE_NAME),bkgMovieFileName=argOpts.bkgMovie.value,cmdLine=sysProcess.env.FFMPEG_BIN||"ffmpeg",scale=opts_parser_js_1.OptsParser.computeScale(report.width,report.height),args=["-r",report.fps.toString(),"-f","image2"];scale||args.push("-s","".concat(report.width,"x").concat(report.height)),args.push("-i",report.framespattern,"-y"),scale&&args.push("-vf","scale=".concat(scale.join("x"))),bkgMovieFileName&&args.push("-vf","movie=".concat(bkgMovieFileName,",hue=s=1[bg];[in]setpts=PTS,scale=-1:-1")+",pad=iw:ih:0:0:color=yellow[m]; [bg][m]overlay=shortest=1:x=0:y=0");var ext=sysPath.extname(movieFileName),codec="";switch(ext){case".mp4":codec="libx264";break;case".webm":codec="libvpx-vp9"}codec&&args.push("-vcodec",codec),args.push(movieFileName),argOpts.moviePre.multipleValue&&(args=__spreadArray(__spreadArray([],argOpts.moviePre.multipleValue,!0),args,!0)),argOpts.moviePost.multipleValue&&(args=__spreadArray(__spreadArray([],args,!0),argOpts.moviePost.multipleValue,!0)),isVerbose&&(console.log("\next: ".concat(ext,"\n")),console.log("cmdLine:[".concat(cmdLine," ").concat(args.join(" "),"]\n\n"))),runSpawn(cmdLine,args,function(){logLevel>consts_js_1.Consts.LL_SILENT&&console.log("Created movie ".concat(movieFileName))})}();break;default:throw"Unknown command ".concat(cmdName)}}var hasMarked,hasHighlightJs,port,allowDirListing}()}catch(err){console.error(err.message||err.toString())}}(Cli||(Cli={}));